from enum import Enum


class PriorityTiket(Enum):
    LOW = 1, "Low"
    MEDIUM = 2, "Medium"
    HIGH = 3, "High"

    def __str__(self) -> str:
        return self.value[1]

    @property
    def id(self) -> int:
        return self.value[0]



from enum import Enum


class TopikTiket(Enum):
    INFO_UT = 1, "Info UT"
    IJAZAH = 8, "Ijazah"
    BEASISWA = 9, "Beasiswa"
    BAHAN_AJAR = 10, "Bahan Ajar"
    ALIH_KREDIT_MATA_KULIAH = 11, "Alih Kredit Mata Kuliah"
    KELULUSAN_YUDISIUM = 13, "Kelulusan/Yudisium"
    NILAI = 14, "Nilai"
    UJIAN = 15, "Ujian"
    BEDA_TANDA_TANGAN = 17, "Beda Tanda Tangan"
    EMAIL_E_CAMPUS = 19, "Email e-campus"
    FORLAP_DIKTI = 21, "Forlap Dikti"
    KTME = 23, "KTME"
    KARIL = 25, "Karil"
    LKAM = 28, "LKAM"
    LATIHAN_MANDIRI = 30, "Latihan Mandiri"
    NILAI_ADA_TP_TDK_TERBIT_DI_LKAM = 33, "Nilai ada tp tdk terbit di LKAM"
    PMKM = 35, "PMKM"
    PEMBAYARAN_SPP = 37, "Pembayaran SPP"
    PEMBEBASAN_MATAKULIAH = 39, "Pembebasan Matakuliah"
    PENAWARAN_KERJASAMA = 41, "Penawaran Kerjasama"
    PENGEMBALIAN_SPP = 43, "Pengembalian SPP"
    PENGGUGURAN_MATAKULIAH = 45, "Pengguguran Matakuliah"
    PENUNDAAN_YUDISIUM = 47, "Penundaan Yudisium"
    PERSETUJUAN_ALIH_KREDIT = 49, "Persetujuan Alih Kredit"
    PERUBAHAN_DATA_PRIBADI = 51, "Perubahan Data Pribadi"
    PINDAH_PROGRAM_STUDI = 53, "Pindah Program Studi"
    PINDAH_UPBJJ = 55, "Pindah UPBJJ"
    PRAKTEK_PRAKTIKUM = 57, "Praktek/Praktikum"
    PROGRAM_SERTIFIKAT = 59, "Program Sertifikat"
    REGISTRASI = 61, "Registrasi"
    RUANG_BACA_VIRTUAL = 63, "Ruang Baca Virtual"
    SUKET_ALUMNI = 65, "Suket Alumni"
    SUKET_KELULUSAN = 67, "Suket Kelulusan"
    SUKET_MAHASISWA_AKTIF = 69, "Suket Mahasiswa Aktif"
    SUKET_PENONAKTIFAN_PINDAH = 71, "Suket Penonaktifan (Pindah)"
    TIDAK_PUAS_NILAI_UAS = 73, "Tidak puas nilai UAS"
    TUTORIAL_TUGAS = 75, "Tutorial/Tugas"
    WISUDA_UPI = 78, "Wisuda/UPI"
    OFFICE_365 = 87, "Office 365"
    TAP_TUGAS_AKHIR_PROGRAM = 91, "TAP (Tugas Akhir Program)"
    FAKULTAS_DAN_PROGRAM_STUDI = 92, "Fakultas dan Program Studi"
    MAHASISWA_UT = 93, "Mahasiswa UT"
    TUGAS_MATAKULIAH_REMEDIASI = 95, "Tugas Matakuliah (Remediasi)"
    UAS_UJIAN_AKHIR_SEMESTER = 97, "UAS (Ujian Akhir Semester)"
    UANG_KULIAH_SPP = 99, "Uang kuliah/SPP"

    def __str__(self) -> str:
        return self.value[1]

    @property
    def id(self) -> int:
        return self.value[0]



import attr
import bleach
import requests
from bs4 import BeautifulSoup, Tag
from requests_toolbelt import MultipartEncoder
from typing import List, Optional, BinaryIO

from hallo_ut.constants import STATUS_TIKET_URL, IGNORED_TAGS
from . import TopikTiket, PriorityTiket


def status_from_page_top(section: Tag) -> bool:
    span: Tag = section.find("span")
    return "OPEN" in span.getText()


class BaseTiket:
    TOPIK = TopikTiket
    PRIORITY = PriorityTiket


@attr.dataclass(slots=True)
class Tiket(BaseTiket):
    status: bool
    judul: str
    nama: str
    email: str
    topik: str
    nomer: str
    pesan: str

    def __str__(self) -> str:
        text = f"Judul: {self.judul}\n"
        text += "Status: "
        if self.status:
            text += "OPEN\n"
        else:
            text += "CLOSE\n"
        text += f"Nama: {self.nama}\n"
        text += f"Email: {self.email}\n"
        text += f"Topik: {self.topik}\n"
        text += f"Nomer Ticket: {self.nomer}\n"
        text += "Pesan:\n\n" + self.clean_pesan
        return text

    def __bool__(self) -> bool:
        return self.status

    @property
    def html(self) -> str:
        text = f'<a href="{self.url}">{self.judul}</a> [(]{self.topik}]\n'
        text += f"Status: {self.status_text} | Nomor: {self.nomer}\n"
        text += f"Nama: {self.nama} | Email: {self.email}\n"
        text += f"Pesan: {self.clean_pesan}"
        return text

    @property
    def status_text(self) -> str:
        return "OPEN" if self.status else "CLOSE"

    @property
    def clean_pesan(self) -> str:
        return bleach.clean(
            text=self.pesan.replace("<br/>", "\n"),
            tags=IGNORED_TAGS,
            strip=True,
        )

    @property
    def url(self) -> str:
        return f"http://hallo-ut.ut.ac.id/status?noticket={self.nomer}"

    @classmethod
    def from_noticket(cls, noticket: str) -> "Tiket":
        params = {"noticket": noticket}
        res = requests.get(STATUS_TIKET_URL, params=params, verify=False)
        if not res.ok or not res.text or "Tiket Tidak Ditemukan" in res.text:
            raise ValueError("Tiket tidak ditemukan atau Hallo-ut tidak bisa dihubungi")
        soup = BeautifulSoup(res.text, "html.parser")
        status = status_from_page_top(soup.find("section", class_="page-top"))
        return cls.from_table(
            table=soup.find("table"),
            status=status,
        )

    @classmethod
    def from_table(cls, table: Tag, status: bool) -> "Tiket":
        tr_tags: Tag = table.find_all("tr")
        judul, nama, email, topik, nomer, pesan = "", "", "", "", "", ""
        nama = tr_tags[0].contents[5].getText()
        judul = tr_tags[0].contents[13].getText()
        email = tr_tags[1].contents[5].getText()
        topik = tr_tags[1].contents[13].getText()
        nomer = tr_tags[2].contents[5].getText()
        pesan = str(tr_tags[2].contents[13])
        return cls(
            status=status,
            judul=judul,
            nama=nama,
            email=email,
            topik=topik,
            nomer=nomer,
            pesan=pesan,
        )

    @classmethod
    def create(
        cls,
        fullname: str,
        email: str,
        nohp: str,
        topik: TopikTiket,
        priority: PriorityTiket,
        subject: str,
        description: str,
        nim: Optional[str] = None,
        userfile: Optional[List[BinaryIO]] = None,
    ) -> "Tiket":
        session = requests.Session()
        session.headers[
            "User-Agent"
        ] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:87.0) Gecko/20100101 Firefox/87.0"
        res = session.get("http://hallo-ut.ut.ac.id/create/ticket")
        if not res.ok or not res.text:
            raise Exception("Hallo-ut tidak dapat dihubungi")
        # TODO : Tambah 3 file gambar ke fields
        fields = {
            "fullname": fullname,
            "email": email,
            "nohp": nohp,
            "topik_id": str(topik.id),
            "priority_id": str(priority.id),
            "subject": subject,
            "Description": description,
            "nim": str(nim or ""),
        }
        m = MultipartEncoder(fields)
        headers = {
            "Content-Type": m.content_type,
            "Referer": res.url,
        }
        res = session.post(
            url="http://hallo-ut.ut.ac.id/ticket/post",
            data=m.to_string(),
            headers=headers,
        )
        if "=" not in res.url:
            raise Exception("Data yang dikirim tidak valid!")
        return cls.from_noticket(res.url.split("=")[-1])



import attr
import bleach
import requests
from bs4 import BeautifulSoup, Tag
from typing import List

from .constants import FAQ_PAGE_URL, IGNORED_TAGS


def parse_contents(contents: Tag, ignored_tags: List[str] = IGNORED_TAGS) -> str:
    text = ""
    for content in contents:
        if isinstance(content, Tag):
            if content.name in ignored_tags:
                text += str(content)
            elif content.name == "ul":
                text += parse_contents(content)
            elif content.name == "br":
                text += "\n"
            elif content.name == "li":
                text += "● " + str(content) + "\n"
            else:
                text += str(content) + "\n"
            continue
        text += str(content)
    return text.strip()


@attr.dataclass(slots=True)
class Faq:
    question: str
    answer: str
    link: str

    def __str__(self) -> str:
        return f"Q: {self.question}\nA: {self.bleached_answer}\nSumber: {self.link}"

    @property
    def bleached_answer(self) -> str:
        return bleach.clean(self.answer, strip=True)

    @property
    def html(self) -> str:
        return (
            f'<b>{self.question}<b/>\n\n{self.answer}\n<a href="{self.link}">Sumber<a/>'
        )

    @classmethod
    def from_panel(cls, panel: Tag, url: str) -> "Faq":
        a = panel.find("a")
        body = panel.find("div", class_="panel-body")
        contents = parse_contents(body)
        return Faq(
            question=a.get_text(strip=True),
            answer=contents or str(body),
            link=url + a["href"],
        )

    @classmethod
    def fetch_faqs(cls, url: str = FAQ_PAGE_URL) -> List["Faq"]:
        results: List["Faq"] = list()
        res = requests.get(url, verify=False)
        if not res.ok:
            pass
        soup = BeautifulSoup(res.text, "html.parser").find(
            "div", {"id": "accordion2", "class": "panel-group"}
        )
        for panel in soup.findAll("div", class_=["panel", "panel-primary"]):
            try:
                results.append(cls.from_panel(panel, url))
            except Exception:
                pass
        return results



import attr
import json
import os

from typing import List, Optional

from . import Faq
from .constants import CACHE_FAQ_FILEPATH


def faqs_to_json(faqs: List["Faq"]) -> List[dict]:
    return [attr.asdict(faq) for faq in faqs]


def save_faqs(faqs: List["Faq"]):
    faqs_json = faqs_to_json(faqs)
    with open(CACHE_FAQ_FILEPATH, "w") as fp:
        json.dump(faqs_json, fp)


def load_faqs() -> Optional[List["Faq"]]:
    if not os.path.isfile(CACHE_FAQ_FILEPATH):
        return None
    results: List[dict] = list()
    with open(CACHE_FAQ_FILEPATH, "r") as fp:
        results = json.load(fp)
    return [Faq(**dat) for dat in results]



import os

FAQ_PAGE_URL = "http://hallo-ut.ut.ac.id/informasi"
STATUS_TIKET_URL = "http://hallo-ut.ut.ac.id/status"
IGNORED_TAGS = ["a", "b", "u", "i", "s", "code", "pre"]
HOME_DIR = os.environ.get("HALLO_UT_HOME") or os.path.join(
    os.environ.get("TYPE_CHECKING") or os.path.expanduser("~/.cache"), "hallo-ut"
)
if not os.path.isdir(HOME_DIR):
    from pathlib import Path

    Path(HOME_DIR).mkdir(parents=True, exist_ok=True)
CACHE_FAQ_FILEPATH = os.path.join(HOME_DIR, "faq.json")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hallo_ut', 'hallo_ut.tiket']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=21.0.0,<23.0.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'bleach>=4.0.0,<6.0.0',
 'fuzzywuzzy>=0.18.0,<0.19.0',
 'requests-toolbelt>=0.9.1,<0.10.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['hallo-ut = hallo_ut.__main__:main']}

setup_kwargs = {
    'name': 'hallo-ut',
    'version': '0.5.3',
    'description': 'SDK Python Layanan Informasi & Bantuan Universitas Terbuka',
    'long_description': '# hallo-ut\n\n[![hallo-ut - PyPi](https://img.shields.io/pypi/v/hallo-ut)](https://pypi.org/project/hallo-ut/)\n[![Supported Python versions](https://img.shields.io/pypi/pyversions/hallo-ut)](https://pypi.org/project/hallo-ut/)\n[![LICENSE](https://img.shields.io/github/license/UnivTerbuka/hallo-ut)](https://github.com/UnivTerbuka/hallo-ut/blob/main/LICENSE)\n[![Code Style - Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nSDK Python Layanan Informasi & Bantuan Universitas Terbuka\n\n## Install\n\nInstall dengan [python](https://www.python.org/)\n\n```bash\npip install --upgrade hallo-ut\n```\n\n## Penggunaan\n\nPenggunaan sederhana\n\n```bash\npython -m hallo-ut "pertanyaan anda atau tiket hallo-ut anda"\n```\n\n## Legal / Hukum\n\nKode ini sama sekali tidak berafiliasi dengan, diizinkan, dipelihara, disponsori atau didukung oleh [Universitas Terbuka](https://ut.ac.id/) atau afiliasi atau anak organisasinya. Ini adalah perangkat lunak yang independen dan tidak resmi. Gunakan dengan risiko Anda sendiri.\n',
    'author': 'hexatester',
    'author_email': 'habibrohman@protonmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import requests
import logging
from fuzzywuzzy import process
from typing import List

from . import Faq, Tiket
from .utils import save_faqs, load_faqs


class HalloUt:
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.faqs = self.load_faqs()
        self.faq_q_dict = dict(enumerate([faq.question for faq in self.faqs]))

    def __call__(self, query: str, limit: int = 10) -> List[Faq]:
        return self.faq(query, limit)

    def faq(self, query: str, limit: int = 10, score_cutoff: int = 60) -> List[Faq]:
        self.logger.debug(f"Mencari faq dengan keyword `{query}`")
        best_faq = process.extractBests(
            query, self.faq_q_dict, score_cutoff=score_cutoff, limit=10
        )
        self.logger.debug(f"Ditemukan {len(best_faq)} kemungkinan jawaban")
        return [self.faqs[z] for (x, y, z) in best_faq] if best_faq else []

    def status_tiket(self, noticket: str) -> Tiket:
        return Tiket.from_noticket(noticket=noticket)

    @staticmethod
    def load_faqs() -> List[Faq]:
        faqs = load_faqs()
        if faqs:
            return faqs
        faqs = Faq.fetch_faqs()
        save_faqs(faqs)
        return faqs



import sys
import re

from argparse import ArgumentParser
from typing import List

from . import HalloUt


def main():
    parser = ArgumentParser(
        prog="hallo-ut",
        description="Layanan Informasi Universitas Terbuka",
    )
    parser.add_argument(
        "pertanyaan",
        type=str,
        nargs="+",
        help="pertanyaan tentang Universitas Terbuka",
    )
    args = parser.parse_args()
    pertanyaan: List[str] = getattr(args, "pertanyaan", list())
    if not pertanyaan:
        return
    hallo = HalloUt()
    match = re.match(r"^(W\d{10}-\d{8})$", " ".join(pertanyaan))
    if match:
        for group in match.groups():
            if not group:
                continue
            tiket = hallo.status_tiket(group)
            print(str(tiket))
        return
    results = hallo.faq(" ".join(pertanyaan))
    if results:
        print(f"Ditemukan {len(results)} jawaban yang mungkin cocok.")
    else:
        print(f"Jawaban tidak ditemukan.")
        return
    for faq in results:
        print("\n")
        print(faq)


if __name__ == "__main__":
    sys.exit(main())
